from faker.providers import BaseProvider

from faker_pokemon.pokemon_data import (
    POKEMON_ABILITIES,
    POKEMON_BERRIES,
    POKEMON_COMPANIES,
    POKEMON_COMPANY_SUFFIXES,
    POKEMON_ITEM_CATEGORIES,
    POKEMON_ITEMS,
    POKEMON_LOCATIONS,
    POKEMON_MOVES,
    POKEMON_NAMES,
    POKEMON_NATURES,
    POKEMON_REGIONS,
    POKEMON_STREET_SUFFIXES,
    POKEMON_TYPES,
)


class PokemonProvider(BaseProvider):
    """A Faker provider that overrides standard methods with Pokemon-themed data."""

    def pokemon_name(self) -> str:
        """Return a random Pokemon name."""
        return self.random_element(POKEMON_NAMES)

    def pokemon_type(self) -> str:
        """Return a random Pokemon type."""
        return self.random_element(POKEMON_TYPES)

    def pokemon_move(self) -> str:
        """Return a random Pokemon move name."""
        return self.random_element(POKEMON_MOVES)

    def pokemon_ability(self) -> str:
        """Return a random Pokemon ability."""
        return self.random_element(POKEMON_ABILITIES)

    def pokemon_nature(self) -> str:
        """Return a random Pokemon nature."""
        return self.random_element(POKEMON_NATURES)

    def pokemon_berry(self) -> str:
        """Return a random Pokemon berry."""
        return self.random_element(POKEMON_BERRIES)

    def company(self) -> str:
        """Return a random Pokemon world company name."""
        return self.random_element(POKEMON_COMPANIES)

    def company_suffix(self) -> str:
        """Return a random Pokemon company suffix."""
        return self.random_element(POKEMON_COMPANY_SUFFIXES)

    def city(self) -> str:
        """Return a random Pokemon location as city."""
        return self.random_element(POKEMON_LOCATIONS)

    def country(self) -> str:
        """Return a random Pokemon region as country."""
        return self.random_element(POKEMON_REGIONS)

    def street_name(self) -> str:
        """Return a Pokemon-themed street name."""
        return (
            f"{self.random_element(POKEMON_NAMES)} "
            f"{self.random_element(POKEMON_STREET_SUFFIXES)}"
        )

    def first_name(self) -> str:
        """Return a random Pokemon name as first name."""
        return self.random_element(POKEMON_NAMES)

    def last_name(self) -> str:
        """Return a random Pokemon type as last name."""
        return self.random_element(POKEMON_TYPES)

    def full_name(self) -> str:
        """Return a Pokemon-themed full name."""
        return f"{self.first_name()} {self.last_name()}"

    def product_name(self) -> str:
        """Return a random Pokemon item as product name."""
        return self.random_element(POKEMON_ITEMS)

    def product_category(self) -> str:
        """Return a random Pokemon item category."""
        return self.random_element(POKEMON_ITEM_CATEGORIES)



import pytest
from faker import Faker

from faker_pokemon import PokemonProvider
from faker_pokemon.pokemon_data import (
    POKEMON_ABILITIES,
    POKEMON_BERRIES,
    POKEMON_COMPANIES,
    POKEMON_COMPANY_SUFFIXES,
    POKEMON_ITEM_CATEGORIES,
    POKEMON_ITEMS,
    POKEMON_LOCATIONS,
    POKEMON_MOVES,
    POKEMON_NAMES,
    POKEMON_NATURES,
    POKEMON_REGIONS,
    POKEMON_STREET_SUFFIXES,
    POKEMON_TYPES,
)


@pytest.fixture
def fake():
    fake = Faker()
    fake.add_provider(PokemonProvider)
    return fake


class TestPokemonName:
    def test_returns_string(self, fake):
        assert isinstance(fake.pokemon_name(), str)

    def test_returns_valid_pokemon(self, fake):
        assert fake.pokemon_name() in POKEMON_NAMES

    def test_seed_reproducibility(self):
        fake1 = Faker()
        fake1.add_provider(PokemonProvider)
        fake1.seed_instance(12345)

        fake2 = Faker()
        fake2.add_provider(PokemonProvider)
        fake2.seed_instance(12345)

        assert fake1.pokemon_name() == fake2.pokemon_name()


class TestPokemonType:
    def test_returns_string(self, fake):
        assert isinstance(fake.pokemon_type(), str)

    def test_returns_valid_type(self, fake):
        assert fake.pokemon_type() in POKEMON_TYPES


class TestPokemonMove:
    def test_returns_string(self, fake):
        assert isinstance(fake.pokemon_move(), str)

    def test_returns_valid_move(self, fake):
        assert fake.pokemon_move() in POKEMON_MOVES


class TestPokemonAbility:
    def test_returns_string(self, fake):
        assert isinstance(fake.pokemon_ability(), str)

    def test_returns_valid_ability(self, fake):
        assert fake.pokemon_ability() in POKEMON_ABILITIES


class TestPokemonNature:
    def test_returns_string(self, fake):
        assert isinstance(fake.pokemon_nature(), str)

    def test_returns_valid_nature(self, fake):
        assert fake.pokemon_nature() in POKEMON_NATURES


class TestPokemonBerry:
    def test_returns_string(self, fake):
        assert isinstance(fake.pokemon_berry(), str)

    def test_returns_valid_berry(self, fake):
        assert fake.pokemon_berry() in POKEMON_BERRIES


class TestCompany:
    def test_company(self, fake):
        assert fake.company() in POKEMON_COMPANIES

    def test_company_suffix(self, fake):
        assert fake.company_suffix() in POKEMON_COMPANY_SUFFIXES


class TestAddress:
    def test_city(self, fake):
        assert fake.city() in POKEMON_LOCATIONS

    def test_country(self, fake):
        assert fake.country() in POKEMON_REGIONS

    def test_street_name(self, fake):
        street = fake.street_name()
        parts = street.rsplit(" ", 1)
        assert len(parts) == 2
        assert parts[0] in POKEMON_NAMES
        assert parts[1] in POKEMON_STREET_SUFFIXES


class TestPerson:
    def test_first_name(self, fake):
        assert fake.first_name() in POKEMON_NAMES

    def test_last_name(self, fake):
        assert fake.last_name() in POKEMON_TYPES

    def test_full_name(self, fake):
        name = fake.full_name()
        parts = name.split(" ", 1)
        assert parts[0] in POKEMON_NAMES
        assert parts[1] in POKEMON_TYPES


class TestCommerce:
    def test_product_name(self, fake):
        assert fake.product_name() in POKEMON_ITEMS

    def test_product_category(self, fake):
        assert fake.product_category() in POKEMON_ITEM_CATEGORIES



# uv inline script metadata (PEP 723)
# /// script
# dependencies = ["httpx"]
# requires-python = ">=3.14"
# ///
"""Fetch Pokemon data from PokeAPI and update pokemon_data.py."""

import textwrap
from pathlib import Path

import httpx

POKEAPI_BASE = "https://pokeapi.co/api/v2"
TARGET = Path(__file__).resolve().parent.parent / "src" / "faker_pokemon" / "pokemon_data.py"
ENDPOINTS: dict[str, dict] = {
    "POKEMON_NAMES": dict(endpoint="pokemon-species", limit=2000, sort_by_id=True),
    "POKEMON_TYPES": dict(endpoint="type", limit=100),
    "POKEMON_MOVES": dict(endpoint="move", limit=2000, titleize=True),
    "POKEMON_REGIONS": dict(endpoint="region", limit=100),
    "POKEMON_LOCATIONS": dict(endpoint="location", limit=1000, titleize=True),
    "POKEMON_ITEMS": dict(endpoint="item", limit=2500, titleize=True),
    "POKEMON_ITEM_CATEGORIES": dict(endpoint="item-category", limit=100, titleize=True),
    "POKEMON_ABILITIES": dict(endpoint="ability", limit=400, titleize=True),
    "POKEMON_NATURES": dict(endpoint="nature", limit=25),
    "POKEMON_BERRIES": dict(endpoint="berry", limit=100),
}
POKEMON_COMPANIES: tuple[str, ...] = (
    "Silph Co.",
    "Devon Corporation",
    "Lysandre Labs",
    "Macro Cosmos",
    "Aether Foundation",
    "Team Rocket Corporation",
    "Pokemon Fan Club",
    "Global Trade Station",
    "Poketch Company",
    "Battle Company",
    "Castelia Cone",
    "LPokemon",
    "Pokestar Studios",
    "Lumiose Press",
    "Mauville Game Corner",
    "Galactic Energy",
    "Future Industries",
    "Rose Tower Corp.",
    "Treasure Town Guild",
    "Ranger Union",
)
POKEMON_COMPANY_SUFFIXES: tuple[str, ...] = (
    "Co.",
    "Corp.",
    "Corporation",
    "Labs",
    "Foundation",
    "Industries",
    "Inc.",
    "Group",
    "Guild",
    "Union",
)
POKEMON_STREET_SUFFIXES: tuple[str, ...] = (
    "Road",
    "Route",
    "Path",
    "Trail",
    "Avenue",
    "Way",
    "Lane",
    "Bridge",
    "Tunnel",
    "Pass",
)


def main() -> None:
    tuples: dict[str, tuple[str, ...]] = {}
    with httpx.Client(timeout=30) as client:
        for var_name, conf in ENDPOINTS.items():
            endpoint = conf["endpoint"]
            limit = conf["limit"]
            sort_by_id = conf.get("sort_by_id", False)
            titleize = conf.get("titleize", False)
            print(f"Fetching {endpoint}...")
            r = client.get(f"{POKEAPI_BASE}/{endpoint}?limit={limit}")
            r.raise_for_status()
            results = r.json()["results"]
            if sort_by_id:
                results.sort(key=lambda s: int(s["url"].rstrip("/").split("/")[-1]))
            tuples[var_name] = tuple(
                item["name"].replace("-", " ").title() if titleize else item["name"].capitalize()
                for item in results
            )
            print(f"  {len(tuples[var_name])} entries")
    tuples["POKEMON_COMPANIES"] = POKEMON_COMPANIES
    tuples["POKEMON_COMPANY_SUFFIXES"] = POKEMON_COMPANY_SUFFIXES
    tuples["POKEMON_STREET_SUFFIXES"] = POKEMON_STREET_SUFFIXES
    content = textwrap.dedent("""\
        \"\"\"Pokemon data for the faker_pokemon provider.

        Auto-generated by scripts/update_data.py from PokeAPI (https://pokeapi.co/).
        Do not edit manually.
        \"\"\"

    """)
    for name, data in tuples.items():
        entries = "\n".join(f'    "{item}",' for item in data)
        content += f"{name}: tuple[str, ...] = (\n{entries}\n)\n\n"
    TARGET.write_text(content)
    print(f"\nWritten to {TARGET}")


if __name__ == "__main__":
    main()
